@@ -1,5 +1,5 @@
 /*
- * "$Id: escp2-channels.c,v 1.62 2006/06/01 23:49:33 rlk Exp $"
+ * "$Id: escp2-channels.c,v 1.62.8.3 2007/12/29 20:42:26 rlk Exp $"
  *
  *   Print plug-in EPSON ESC/P2 driver for the GIMP.
  *
@@ -285,352 +285,380 @@ static hue_curve_t picturemate_blue_curve =
 
 static const physical_subchannel_t standard_black_subchannels[] =
 {
-  { 0, -1, 0, "BlackDensity", NULL }
+  { 0, -1, 0, 0, "BlackDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(standard_black);
 
 static const physical_subchannel_t f360_black_subchannels[] =
 {
-  { 0, 0, 1, "BlackDensity", NULL }
+  { 0, 0, 1, 0, "BlackDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(f360_black);
 
 static const physical_subchannel_t x80_black_subchannels[] =
 {
-  { 0, -1, 48, "BlackDensity", NULL }
+  { 0, -1, 48, 0, "BlackDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(x80_black);
 
 static const physical_subchannel_t c80_black_subchannels[] =
 {
-  { 0, -1, 0, "BlackDensity", NULL }
+  { 0, -1, 0, 0, "BlackDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(c80_black);
 
+static const physical_subchannel_t c120_black_subchannels[] =
+{
+  { 0, 4, 0, 2, "BlackDensity", NULL, { 0, 0x40 } }
+};
+
+DECLARE_INK_CHANNEL(c120_black);
+
 static const physical_subchannel_t c64_black_subchannels[] =
 {
-  { 0, -1, 0, "BlackDensity", NULL }
+  { 0, -1, 0, 0, "BlackDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(c64_black);
 
 static const physical_subchannel_t standard_cyan_subchannels[] =
 {
-  { 2, -1, 0, "CyanDensity", NULL }
+  { 2, -1, 0, 0, "CyanDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(standard_cyan);
 
 static const physical_subchannel_t f360_standard_cyan_subchannels[] =
 {
-  { 2, -1, 1, "CyanDensity", NULL }
+  { 2, -1, 1, 0, "CyanDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(f360_standard_cyan);
 
 static const physical_subchannel_t r800_cyan_subchannels[] =
 {
-  { 2, -1, 1, "CyanDensity", NULL }
+  { 2, -1, 1, 0, "CyanDensity", NULL }
 };
 
 DECLARE_EXTENDED_INK_CHANNEL(r800_cyan);
 
 static const physical_subchannel_t picturemate_cyan_subchannels[] =
 {
-  { 2, -1, 0, "CyanDensity", NULL }
+  { 2, -1, 0, 0, "CyanDensity", NULL }
 };
 
 DECLARE_EXTENDED_INK_CHANNEL(picturemate_cyan);
 
 static const physical_subchannel_t cx3650_standard_cyan_subchannels[] =
 {
-  { 2, -1, 2, "CyanDensity", NULL }
+  { 2, -1, 2, 0, "CyanDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(cx3650_standard_cyan);
 
 static const physical_subchannel_t x80_cyan_subchannels[] =
 {
-  { 2, -1, 96, "CyanDensity", NULL }
+  { 2, -1, 96, 0, "CyanDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(x80_cyan);
 
 static const physical_subchannel_t c80_cyan_subchannels[] =
 {
-  { 2, -1, 0, "CyanDensity", NULL }
+  { 2, -1, 0, 0, "CyanDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(c80_cyan);
 
+static const physical_subchannel_t c120_cyan_subchannels[] =
+{
+  { 2, -1, 0, 0, "CyanDensity", NULL }
+};
+
+DECLARE_INK_CHANNEL(c120_cyan);
+
 static const physical_subchannel_t c64_cyan_subchannels[] =
 {
-  { 2, -1, 0, "CyanDensity", NULL }
+  { 2, -1, 0, 0, "CyanDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(c64_cyan);
 
 static const physical_subchannel_t standard_magenta_subchannels[] =
 {
-  { 1, -1, 0, "MagentaDensity", NULL }
+  { 1, -1, 0, 0, "MagentaDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(standard_magenta);
 
 static const physical_subchannel_t r800_magenta_subchannels[] =
 {
-  { 1, -1, 0, "MagentaDensity", NULL }
+  { 1, -1, 0, 0, "MagentaDensity", NULL }
 };
 
 DECLARE_EXTENDED_INK_CHANNEL(r800_magenta);
 
 static const physical_subchannel_t picturemate_magenta_subchannels[] =
 {
-  { 1, -1, 0, "MagentaDensity", NULL }
+  { 1, -1, 0, 0, "MagentaDensity", NULL }
 };
 
 DECLARE_EXTENDED_INK_CHANNEL(picturemate_magenta);
 
 static const physical_subchannel_t f360_standard_magenta_subchannels[] =
 {
-  { 1, -1, 1, "MagentaDensity", NULL }
+  { 1, -1, 1, 0, "MagentaDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(f360_standard_magenta);
 
 static const physical_subchannel_t x80_magenta_subchannels[] =
 {
-  { 1, -1, 48, "MagentaDensity", NULL }
+  { 1, -1, 48, 0, "MagentaDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(x80_magenta);
 
 static const physical_subchannel_t c80_magenta_subchannels[] =
 {
-  { 1, -1, 120, "MagentaDensity", NULL }
+  { 1, -1, 120, 0, "MagentaDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(c80_magenta);
 
+static const physical_subchannel_t c120_magenta_subchannels[] =
+{
+  { 1, -1, 120, 0, "MagentaDensity", NULL }
+};
+
+DECLARE_INK_CHANNEL(c120_magenta);
+
 static const physical_subchannel_t c64_magenta_subchannels[] =
 {
-  { 1, -1, 90, "MagentaDensity", NULL }
+  { 1, -1, 90, 0, "MagentaDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(c64_magenta);
 
 static const physical_subchannel_t standard_yellow_subchannels[] =
 {
-  { 4, -1, 0, "YellowDensity", NULL }
+  { 4, -1, 0, 0, "YellowDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(standard_yellow);
 
 static const physical_subchannel_t x80_yellow_subchannels[] =
 {
-  { 4, -1, 0, "YellowDensity", NULL }
+  { 4, -1, 0, 0, "YellowDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(x80_yellow);
 
 static const physical_subchannel_t c80_yellow_subchannels[] =
 {
-  { 4, -1, 240, "YellowDensity", NULL }
+  { 4, -1, 240, 0, "YellowDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(c80_yellow);
 
+static const physical_subchannel_t c120_yellow_subchannels[] =
+{
+  { 4, -1, 240, 0, "YellowDensity", NULL }
+};
+
+DECLARE_INK_CHANNEL(c120_yellow);
+
 static const physical_subchannel_t c64_yellow_subchannels[] =
 {
-  { 4, -1, 180, "YellowDensity", NULL }
+  { 4, -1, 180, 0, "YellowDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(c64_yellow);
 
 static const physical_subchannel_t f360_standard_yellow_subchannels[] =
 {
-  { 4, -1, 1, "YellowDensity", NULL }
+  { 4, -1, 1, 0, "YellowDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(f360_standard_yellow);
 
 static const physical_subchannel_t r800_yellow_subchannels[] =
 {
-  { 4, -1, 1, "YellowDensity", NULL }
+  { 4, -1, 1, 0, "YellowDensity", NULL }
 };
 
 DECLARE_EXTENDED_INK_CHANNEL(r800_yellow);
 
 static const physical_subchannel_t picturemate_yellow_subchannels[] =
 {
-  { 4, -1, 0, "YellowDensity", NULL }
+  { 4, -1, 0, 0, "YellowDensity", NULL }
 };
 
 DECLARE_EXTENDED_INK_CHANNEL(picturemate_yellow);
 
 static const physical_subchannel_t r800_red_subchannels[] =
 {
-  { 7, -1, 0, "RedDensity", NULL }
+  { 7, -1, 0, 0, "RedDensity", NULL }
 };
 
 DECLARE_EXTENDED_INK_CHANNEL(r800_red);
 
 static const physical_subchannel_t picturemate_red_subchannels[] =
 {
-  { 7, -1, 0, "RedDensity", NULL }
+  { 7, -1, 0, 0, "RedDensity", NULL }
 };
 
 DECLARE_EXTENDED_INK_CHANNEL(picturemate_red);
 
 static const physical_subchannel_t r800_blue_subchannels[] =
 {
-  { 8, -1, 1, "BlueDensity", NULL }
+  { 8, -1, 1, 0, "BlueDensity", NULL }
 };
 
 DECLARE_EXTENDED_INK_CHANNEL(r800_blue);
 
 static const physical_subchannel_t picturemate_blue_subchannels[] =
 {
-  { 8, -1, 1, "BlueDensity", NULL }
+  { 8, -1, 1, 0, "BlueDensity", NULL }
 };
 
 DECLARE_EXTENDED_INK_CHANNEL(picturemate_blue);
 
 static const physical_subchannel_t standard_gloss_subchannels[] =
 {
-  { 9, -1, 0, "GlossDensity", NULL }
+  { 9, -1, 0, 0, "GlossDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(standard_gloss);
 
 static const physical_subchannel_t f360_gloss_subchannels[] =
 {
-  { 9, -1, 1, "GlossDensity", NULL }
+  { 9, -1, 1, 0, "GlossDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(f360_gloss);
 
 static const physical_subchannel_t standard_photo_black_subchannels[] =
 {
-  { 0, 4, 0, "BlackDensity", NULL }
+  { 0, 4, 0, 0, "BlackDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(standard_photo_black);
 
 static const physical_subchannel_t f360_photo_black_subchannels[] =
 {
-  { 0, 4, 1, "BlackDensity", NULL }
+  { 0, 4, 1, 0, "BlackDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(f360_photo_black);
 
 static const physical_subchannel_t r800_matte_black_subchannels[] =
 {
-  { 0, 0, 0, "BlackDensity", NULL }
+  { 0, 0, 0, 0, "BlackDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(r800_matte_black);
 
 static const physical_subchannel_t photo_black_subchannels[] =
 {
-  { 0, 0, 0, "BlackDensity", NULL }
+  { 0, 0, 0, 0, "BlackDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(photo_black);
 
 static const physical_subchannel_t extended_black_subchannels[] =
 {
-  { 0, 1, 0, "BlackDensity", NULL }
+  { 0, 1, 0, 0, "BlackDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(extended_black);
 
 static const physical_subchannel_t f360_extended_black_subchannels[] =
 {
-  { 0, 1, 1, "BlackDensity", NULL }
+  { 0, 1, 1, 0, "BlackDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(f360_extended_black);
 
 static const physical_subchannel_t photo_cyan_subchannels[] =
 {
-  { 2, 0, 0, "CyanDensity", NULL },
-  { 2, 1, 0, "CyanDensity", "LightCyanTransition" }
+  { 2, 0, 0, 0, "CyanDensity", NULL },
+  { 2, 1, 0, 0, "CyanDensity", "LightCyanTransition" }
 };
 
 DECLARE_INK_CHANNEL(photo_cyan);
 
 static const physical_subchannel_t extended_cyan_subchannels[] =
 {
-  { 2, 1, 0, "CyanDensity", NULL }
+  { 2, 1, 0, 0, "CyanDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(extended_cyan);
 
 static const physical_subchannel_t f360_extended_cyan_subchannels[] =
 {
-  { 2, 1, 1, "CyanDensity", NULL }
+  { 2, 1, 1, 0, "CyanDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(f360_extended_cyan);
 
 static const physical_subchannel_t photo_magenta_subchannels[] =
 {
-  { 1, 0, 0, "MagentaDensity", NULL },
-  { 1, 1, 0, "MagentaDensity", "LightMagentaTransition" }
+  { 1, 0, 0, 0, "MagentaDensity", NULL },
+  { 1, 1, 0, 0, "MagentaDensity", "LightMagentaTransition" }
 };
 
 DECLARE_INK_CHANNEL(photo_magenta);
 
 static const physical_subchannel_t extended_magenta_subchannels[] =
 {
-  { 1, 1, 0, "MagentaDensity", NULL }
+  { 1, 1, 0, 0, "MagentaDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(extended_magenta);
 
 static const physical_subchannel_t f360_extended_magenta_subchannels[] =
 {
-  { 1, 1, 1, "MagentaDensity", NULL }
+  { 1, 1, 1, 0, "MagentaDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(f360_extended_magenta);
 
 static const physical_subchannel_t photo_yellow_subchannels[] =
 {
-  { 4, 0, 0, "YellowDensity", NULL }
+  { 4, 0, 0, 0, "YellowDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(photo_yellow);
 
 static const physical_subchannel_t f360_photo_yellow_subchannels[] =
 {
-  { 4, 0, 1, "YellowDensity", NULL }
+  { 4, 0, 1, 0, "YellowDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(f360_photo_yellow);
 
 static const physical_subchannel_t j_extended_yellow_subchannels[] =
 {
-  { 4, 2, 0, "YellowDensity", NULL }
+  { 4, 2, 0, 0, "YellowDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(j_extended_yellow);
 
 static const physical_subchannel_t extended_photo3_black_subchannels[] =
 {
-  { 0, 3, 0, "BlackDensity", NULL }
+  { 0, 3, 0, 0, "BlackDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(extended_photo3_black);
 
 static const physical_subchannel_t f360_extended_photo3_black_subchannels[] =
 {
-  { 0, 3, 1, "BlackDensity", NULL }
+  { 0, 3, 1, 0, "BlackDensity", NULL }
 };
 
 DECLARE_INK_CHANNEL(f360_extended_photo3_black);
@@ -638,136 +666,204 @@ DECLARE_INK_CHANNEL(f360_extended_photo3_black);
 /* For Japanese 7-color printers, with dark yellow */
 static const physical_subchannel_t photo2_yellow_subchannels[] =
 {
-  { 4, 2, 0, "YellowDensity", NULL },
-  { 4, 0, 0, "YellowDensity", "DarkYellowTransition" }
+  { 4, 2, 0, 0, "YellowDensity", NULL },
+  { 4, 0, 0, 0, "YellowDensity", "DarkYellowTransition" }
 };
 
 DECLARE_INK_CHANNEL(photo2_yellow);
 
 static const physical_subchannel_t f360_photo2_yellow_subchannels[] =
 {
-  { 4, 2, 0, "YellowDensity", NULL },
-  { 4, 0, 1, "YellowDensity", "DarkYellowTransition" }
+  { 4, 2, 0, 0, "YellowDensity", NULL },
+  { 4, 0, 1, 0, "YellowDensity", "DarkYellowTransition" }
 };
 
 DECLARE_INK_CHANNEL(f360_photo2_yellow);
 
 static const physical_subchannel_t photo2_black_subchannels[] =
 {
-  { 0, 0, 0, "BlackDensity", NULL },
-  { 0, 1, 0, "BlackDensity", "GrayTransition" }
+  { 0, 0, 0, 0, "BlackDensity", NULL },
+  { 0, 1, 0, 0, "BlackDensity", "GrayTransition" }
 };
 
 DECLARE_INK_CHANNEL(photo2_black);
 
 static const physical_subchannel_t f360_photo2_black_subchannels[] =
 {
-  { 0, 0, 1, "BlackDensity", NULL },
-  { 0, 1, 0, "BlackDensity", "GrayTransition" }
+  { 0, 0, 1, 0, "BlackDensity", NULL },
+  { 0, 1, 0, 0, "BlackDensity", "GrayTransition" }
 };
 
 DECLARE_INK_CHANNEL(f360_photo2_black);
 
 static const physical_subchannel_t photo3_black_subchannels[] =
 {
-  { 0, 0, 0, "BlackDensity", NULL },
-  { 0, 1, 0, "BlackDensity", "DarkGrayTransition" },
-  { 0, 3, 0, "BlackDensity", "LightGrayTransition" }
+  { 0, 0, 0, 0, "BlackDensity", NULL },
+  { 0, 1, 0, 0, "BlackDensity", "DarkGrayTransition" },
+  { 0, 3, 0, 0, "BlackDensity", "LightGrayTransition" }
 };
 
 DECLARE_INK_CHANNEL(photo3_black);
 
 static const physical_subchannel_t f360_photo3_black_subchannels[] =
 {
-  { 0, 0, 1, "BlackDensity", NULL },
-  { 0, 1, 0, "BlackDensity", "DarkGrayTransition" },
-  { 0, 3, 1, "BlackDensity", "LightGrayTransition" }
+  { 0, 0, 1, 0, "BlackDensity", NULL },
+  { 0, 1, 0, 0, "BlackDensity", "DarkGrayTransition" },
+  { 0, 3, 1, 0, "BlackDensity", "LightGrayTransition" }
 };
 
 DECLARE_INK_CHANNEL(f360_photo3_black);
 
 static const physical_subchannel_t quadtone_subchannels[] =
 {
-  { 0, -1, 0, "BlackDensity", NULL },
-  { 2, -1, 0, "BlackDensity", "Gray3Transition" },
-  { 1, -1, 0, "BlackDensity", "Gray2Transition" },
-  { 4, -1, 0, "BlackDensity", "Gray1Transition" },
+  { 0, -1, 0, 0, "BlackDensity", NULL },
+  { 2, -1, 0, 0, "BlackDensity", "Gray3Transition" },
+  { 1, -1, 0, 0, "BlackDensity", "Gray2Transition" },
+  { 4, -1, 0, 0, "BlackDensity", "Gray1Transition" },
 };
 
 DECLARE_INK_CHANNEL(quadtone);
 
 static const physical_subchannel_t c80_quadtone_subchannels[] =
 {
-  { 0, -1, 0, "BlackDensity", NULL },
-  { 2, -1, 0, "BlackDensity", "Gray3Transition" },
-  { 1, -1, 120, "BlackDensity", "Gray2Transition" },
-  { 4, -1, 240, "BlackDensity", "Gray1Transition" },
+  { 0, -1, 0, 0, "BlackDensity", NULL },
+  { 2, -1, 0, 0, "BlackDensity", "Gray3Transition" },
+  { 1, -1, 120, 0, "BlackDensity", "Gray2Transition" },
+  { 4, -1, 240, 0, "BlackDensity", "Gray1Transition" },
 };
 
 DECLARE_INK_CHANNEL(c80_quadtone);
 
+static const physical_subchannel_t c120_quadtone_subchannels[] =
+{
+  { 0, 4, 0, 0, "BlackDensity", NULL },
+  { 2, -1, 0, 0, "BlackDensity", "Gray3Transition" },
+  { 1, -1, 120, 0, "BlackDensity", "Gray2Transition" },
+  { 4, -1, 240, 0, "BlackDensity", "Gray1Transition" },
+};
+
+DECLARE_INK_CHANNEL(c120_quadtone);
+
 static const physical_subchannel_t c64_quadtone_subchannels[] =
 {
-  { 0, -1, 0, "BlackDensity", NULL },
-  { 2, -1, 0, "BlackDensity", "Gray3Transition" },
-  { 1, -1, 90, "BlackDensity", "Gray2Transition" },
-  { 4, -1, 180, "BlackDensity", "Gray1Transition" },
+  { 0, -1, 0, 0, "BlackDensity", NULL },
+  { 2, -1, 0, 0, "BlackDensity", "Gray3Transition" },
+  { 1, -1, 90, 0, "BlackDensity", "Gray2Transition" },
+  { 4, -1, 180, 0, "BlackDensity", "Gray1Transition" },
 };
 
 DECLARE_INK_CHANNEL(c64_quadtone);
 
 static const physical_subchannel_t f360_quadtone_subchannels[] =
 {
-  { 0, -1, 0, "BlackDensity", NULL },
-  { 2, -1, 1, "BlackDensity", "Gray3Transition" },
-  { 1, -1, 1, "BlackDensity", "Gray2Transition" },
-  { 4, -1, 0, "BlackDensity", "Gray1Transition" },
+  { 0, -1, 0, 0, "BlackDensity", NULL },
+  { 2, -1, 1, 0, "BlackDensity", "Gray3Transition" },
+  { 1, -1, 1, 0, "BlackDensity", "Gray2Transition" },
+  { 4, -1, 0, 0, "BlackDensity", "Gray1Transition" },
 };
 
 DECLARE_INK_CHANNEL(f360_quadtone);
 
 static const physical_subchannel_t cx3650_quadtone_subchannels[] =
 {
-  { 0, -1, 0, "BlackDensity", NULL },
-  { 2, -1, 2, "BlackDensity", "Gray3Transition" },
-  { 1, -1, 1, "BlackDensity", "Gray2Transition" },
-  { 4, -1, 0, "BlackDensity", "Gray1Transition" },
+  { 0, -1, 0, 0, "BlackDensity", NULL },
+  { 2, -1, 2, 0, "BlackDensity", "Gray3Transition" },
+  { 1, -1, 1, 0, "BlackDensity", "Gray2Transition" },
+  { 4, -1, 0, 0, "BlackDensity", "Gray1Transition" },
 };
 
 DECLARE_INK_CHANNEL(cx3650_quadtone);
 
 static const physical_subchannel_t f360_photo_cyan_subchannels[] =
 {
-  { 2, 0, 1, "CyanDensity", NULL },
-  { 2, 1, 0, "CyanDensity", "LightCyanTransition" }
+  { 2, 0, 1, 0, "CyanDensity", NULL },
+  { 2, 1, 0, 0, "CyanDensity", "LightCyanTransition" }
 };
 
 DECLARE_INK_CHANNEL(f360_photo_cyan);
 
 static const physical_subchannel_t f360x_photo_cyan_subchannels[] =
 {
-  { 2, 0, 0, "CyanDensity", NULL },
-  { 2, 1, 1, "CyanDensity", "LightCyanTransition" }
+  { 2, 0, 0, 0, "CyanDensity", NULL },
+  { 2, 1, 1, 0, "CyanDensity", "LightCyanTransition" }
 };
 
 DECLARE_INK_CHANNEL(f360x_photo_cyan);
 
 static const physical_subchannel_t f360_photo_magenta_subchannels[] =
 {
-  { 1, 0, 1, "MagentaDensity", NULL },
-  { 1, 1, 0, "MagentaDensity", "LightMagentaTransition" }
+  { 1, 0, 1, 0, "MagentaDensity", NULL },
+  { 1, 1, 0, 0, "MagentaDensity", "LightMagentaTransition" }
 };
 
 DECLARE_INK_CHANNEL(f360_photo_magenta);
 
 static const physical_subchannel_t f360x_photo_magenta_subchannels[] =
 {
-  { 1, 0, 0, "MagentaDensity", NULL },
-  { 1, 1, 1, "MagentaDensity", "LightMagentaTransition" }
+  { 1, 0, 0, 0, "MagentaDensity", NULL },
+  { 1, 1, 1, 0, "MagentaDensity", "LightMagentaTransition" }
 };
 
 DECLARE_INK_CHANNEL(f360x_photo_magenta);
 
+static const physical_subchannel_t claria_black_subchannels[] =
+{
+  { 0, 0, 0, 0, "BlackDensity", NULL },
+};
+
+DECLARE_INK_CHANNEL(claria_black);
+
+static const physical_subchannel_t claria_yellow_subchannels[] =
+{
+  { 4, 0, 2, 0, "YellowDensity", NULL },
+};
+
+DECLARE_INK_CHANNEL(claria_yellow);
+
+static const physical_subchannel_t claria_cyan_subchannels[] =
+{
+  { 2, 0, 0, 0, "CyanDensity", NULL },
+};
+
+DECLARE_INK_CHANNEL(claria_cyan);
+
+static const physical_subchannel_t claria_photo_cyan_subchannels[] =
+{
+  { 2, 0, 0, 0, "CyanDensity", NULL },
+  { 2, 1, 2, 0, "CyanDensity", "LightCyanTransition" },
+};
+
+DECLARE_INK_CHANNEL(claria_photo_cyan);
+
+static const physical_subchannel_t extended_claria_cyan_subchannels[] =
+{
+  { 2, 1, 2, 0, "CyanDensity", "LightCyanTransition" },
+};
+
+DECLARE_INK_CHANNEL(extended_claria_cyan);
+
+static const physical_subchannel_t claria_magenta_subchannels[] =
+{
+  { 1, 0, 2, 0, "MagentaDensity", NULL },
+};
+
+DECLARE_INK_CHANNEL(claria_magenta);
+
+static const physical_subchannel_t claria_photo_magenta_subchannels[] =
+{
+  { 1, 0, 2, 0, "MagentaDensity", NULL },
+  { 1, 1, 0, 0, "MagentaDensity", "LightMagentaTransition" },
+};
+
+DECLARE_INK_CHANNEL(claria_photo_magenta);
+
+static const physical_subchannel_t extended_claria_magenta_subchannels[] =
+{
+  { 1, 1, 0, 0, "MagentaDensity", "LightMagentaTransition" },
+};
+
+DECLARE_INK_CHANNEL(extended_claria_magenta);
+
 
 #define DECLARE_CHANNEL_SET(name)			\
 static const channel_set_t name##_channel_set =		\
@@ -815,7 +911,7 @@ static const ink_channel_t *const standard_black_channels[] =
 
 DECLARE_CHANNEL_SET(standard_black);
 
-const escp2_inkname_t stpi_escp2_default_black_inkset =
+static const escp2_inkname_t stpi_escp2_default_black_inkset =
 {
   "Gray", N_("Grayscale"), INKSET_CMYK,
   &standard_black_channel_set
@@ -828,7 +924,7 @@ static const ink_channel_t *const standard_photo_black_channels[] =
 
 DECLARE_CHANNEL_SET(standard_photo_black);
 
-const escp2_inkname_t stpi_escp2_default_photo_black_inkset =
+static const escp2_inkname_t stpi_escp2_default_photo_black_inkset =
 {
   "Gray", N_("Grayscale"), INKSET_CMYK,
   &standard_photo_black_channel_set
@@ -842,7 +938,7 @@ static const ink_channel_t *const standard_photo_gloss_black_channels[] =
 DECLARE_CHANNEL_SET(standard_photo_gloss_black);
 DECLARE_AUX_CHANNEL_SET(standard_photo_black, standard_gloss);
 
-const escp2_inkname_t stpi_escp2_default_photo_gloss_black_inkset =
+static const escp2_inkname_t stpi_escp2_default_photo_gloss_black_inkset =
 {
   "GrayG", N_("Grayscale"), INKSET_CMYK,
   &standard_photo_black_standard_gloss_channel_set
@@ -953,6 +1049,19 @@ static const escp2_inkname_t c80_generic_quadtone_inkset =
   &c80_quadtone_channel_set
 };
 
+static const ink_channel_t *const c120_quadtone_channels[] =
+{
+  &c120_quadtone_channel
+};
+
+DECLARE_CHANNEL_SET(c120_quadtone);
+
+static const escp2_inkname_t c120_generic_quadtone_inkset =
+{
+  "Quadtone", N_("Quadtone"), INKSET_QUADTONE,
+  &c120_quadtone_channel_set
+};
+
 static const ink_channel_t *const c64_quadtone_channels[] =
 {
   &c64_quadtone_channel
@@ -1044,6 +1153,20 @@ static const escp2_inkname_t c80_three_color_composite_inkset =
   &c80_cmy_channel_set
 };
 
+static const ink_channel_t *const c120_cmy_channels[] =
+{
+  NULL, &c120_cyan_channel,
+  &c120_magenta_channel, &c120_yellow_channel
+};
+
+DECLARE_CHANNEL_SET(c120_cmy);
+
+static const escp2_inkname_t c120_three_color_composite_inkset =
+{
+  "RGB", N_("Three Color Composite"), INKSET_CMYK,
+  &c120_cmy_channel_set
+};
+
 static const ink_channel_t *const c64_cmy_channels[] =
 {
   NULL, &c64_cyan_channel,
@@ -1130,6 +1253,20 @@ static const escp2_inkname_t three_color_r2400_composite_inkset =
   &r2400_cmy_channel_set
 };
 
+static const ink_channel_t *const claria_cmy_channels[] =
+{
+  NULL, &claria_cyan_channel,
+  &claria_magenta_channel, &claria_yellow_channel
+};
+
+DECLARE_CHANNEL_SET(claria_cmy);
+
+static const escp2_inkname_t claria_three_color_composite_inkset =
+{
+  "RGB", N_("Three Color Composite"), INKSET_CMYK,
+  &claria_cmy_channel_set
+};
+
 /*
  ****************************************************************
  *                                                              *
@@ -1246,6 +1383,20 @@ static const escp2_inkname_t c80_four_color_standard_inkset =
   &c80_cmyk_channel_set
 };
 
+static const ink_channel_t *const c120_cmyk_channels[] =
+{
+  &c120_black_channel, &c120_cyan_channel,
+  &c120_magenta_channel, &c120_yellow_channel
+};
+
+DECLARE_CHANNEL_SET(c120_cmyk);
+
+static const escp2_inkname_t c120_four_color_standard_inkset =
+{
+  "CMYK", N_("Four Color Standard"), INKSET_CMYK,
+  &c120_cmyk_channel_set
+};
+
 static const ink_channel_t *const c64_cmyk_channels[] =
 {
   &c64_black_channel, &c64_cyan_channel,
@@ -1288,6 +1439,20 @@ static const escp2_inkname_t cx3650_four_color_standard_inkset =
   &cx3650_cmyk_channel_set
 };
 
+static const ink_channel_t *const claria_cmyk_channels[] =
+{
+  &claria_black_channel, &claria_cyan_channel,
+  &claria_magenta_channel, &claria_yellow_channel
+};
+
+DECLARE_CHANNEL_SET(claria_cmyk);
+
+static const escp2_inkname_t claria_four_color_standard_inkset =
+{
+  "CMYK", N_("Four Color Standard"), INKSET_CMYK,
+  &claria_cmyk_channel_set
+};
+
 
 /*
  ****************************************************************
@@ -1339,6 +1504,20 @@ static const escp2_inkname_t five_color_photo3_inkset =
   &five_color_photo3_channel_set
 };
 
+static const ink_channel_t *const claria_ccmmy_channels[] =
+{
+  NULL, &claria_photo_cyan_channel,
+  &claria_photo_magenta_channel, &claria_yellow_channel
+};
+
+DECLARE_CHANNEL_SET(claria_ccmmy);
+
+static const escp2_inkname_t claria_five_color_photo_composite_inkset =
+{
+  "PhotoCMY", N_("Five Color Photo Composite"), INKSET_CcMmYK,
+  &claria_ccmmy_channel_set
+};
+
 
 /*
  ****************************************************************
@@ -1390,6 +1569,20 @@ static const escp2_inkname_t six_color_photo3_inkset =
   &six_color_photo3_channel_set
 };
 
+static const ink_channel_t *const claria_ccmmyk_channels[] =
+{
+  &claria_black_channel, &claria_photo_cyan_channel,
+  &claria_photo_magenta_channel, &claria_yellow_channel
+};
+
+DECLARE_CHANNEL_SET(claria_ccmmyk);
+
+static const escp2_inkname_t claria_six_color_photo_inkset =
+{
+  "PhotoCMYK", N_("Six Color Photo"), INKSET_CcMmYK,
+  &claria_ccmmyk_channel_set
+};
+
 /*
  ****************************************************************
  *                                                              *
@@ -1684,6 +1877,18 @@ static const escp2_inkname_t one_color_extended_inkset =
   &one_color_extended_channel_set
 };
 
+static const ink_channel_t *const c120_one_color_extended_channels[] =
+{
+  &c120_black_channel
+};
+DECLARE_CHANNEL_SET(c120_one_color_extended);
+
+static const escp2_inkname_t c120_one_color_extended_inkset =
+{
+  "PhysicalBlack", N_("One Color Raw"), INKSET_EXTENDED,
+  &c120_one_color_extended_channel_set
+};
+
 static const escp2_inkname_t one_color_photo_extended_inkset =
 {
   "PhysicalBlack", N_("One Color Raw"), INKSET_EXTENDED,
@@ -1789,6 +1994,19 @@ static const escp2_inkname_t c80_three_color_extended_inkset =
   &c80_three_color_extended_channel_set
 };
 
+static const ink_channel_t *const c120_three_color_extended_channels[] =
+{
+  &c120_cyan_channel, &c120_magenta_channel, &c120_yellow_channel
+};
+
+DECLARE_CHANNEL_SET(c120_three_color_extended);
+
+static const escp2_inkname_t c120_three_color_extended_inkset =
+{
+  "PhysicalCMY", N_("Three Color Raw"), INKSET_EXTENDED,
+  &c120_three_color_extended_channel_set
+};
+
 static const ink_channel_t *const c64_three_color_extended_channels[] =
 {
   &c64_cyan_channel, &c64_magenta_channel, &c64_yellow_channel
@@ -1830,6 +2048,19 @@ static const escp2_inkname_t cx3650_three_color_extended_inkset =
   &cx3650_three_color_extended_channel_set
 };
 
+static const ink_channel_t *const claria_three_color_extended_channels[] =
+{
+  &claria_cyan_channel, &claria_magenta_channel, &claria_yellow_channel
+};
+
+DECLARE_CHANNEL_SET(claria_three_color_extended);
+
+static const escp2_inkname_t claria_three_color_extended_inkset =
+{
+  "PhysicalCMY", N_("Three Color Raw"), INKSET_EXTENDED,
+  &claria_three_color_extended_channel_set
+};
+
 
 static const escp2_inkname_t four_color_extended_inkset =
 {
@@ -1855,6 +2086,12 @@ static const escp2_inkname_t c80_four_color_extended_inkset =
   &c80_cmyk_channel_set
 };
 
+static const escp2_inkname_t c120_four_color_extended_inkset =
+{
+  "PhysicalCMYK", N_("Four Color Raw"), INKSET_EXTENDED,
+  &c120_cmyk_channel_set
+};
+
 static const escp2_inkname_t c64_four_color_extended_inkset =
 {
   "PhysicalCMYK", N_("Four Color Raw"), INKSET_EXTENDED,
@@ -1873,6 +2110,12 @@ static const escp2_inkname_t cx3650_four_color_extended_inkset =
   &cx3650_cmyk_channel_set
 };
 
+static const escp2_inkname_t claria_four_color_extended_inkset =
+{
+  "PhysicalCMYK", N_("Four Color Raw"), INKSET_EXTENDED,
+  &claria_cmyk_channel_set
+};
+
 static const escp2_inkname_t four_color_r800_extended_inkset =
 {
   "PhysicalCMYKGloss", N_("Four Color Raw"), INKSET_EXTENDED,
@@ -1914,6 +2157,20 @@ static const escp2_inkname_t f360_five_color_extended_inkset =
   &f360_five_color_extended_channel_set
 };
 
+static const ink_channel_t *const claria_five_color_extended_channels[] =
+{
+  &claria_cyan_channel, &extended_claria_cyan_channel,
+  &claria_magenta_channel, &extended_claria_magenta_channel,
+  &claria_yellow_channel
+};
+DECLARE_CHANNEL_SET(claria_five_color_extended);
+
+static const escp2_inkname_t claria_five_color_extended_inkset =
+{
+  "PhysicalCcMmYK", N_("Six Color Raw"), INKSET_EXTENDED,
+  &claria_five_color_extended_channel_set
+};
+
 
 static const ink_channel_t *const six_color_extended_channels[] =
 {
@@ -1963,6 +2220,22 @@ static const escp2_inkname_t six_color_r800_photo_gloss_extended_inkset =
   &six_color_r800_photo_gloss_channel_set
 };
 
+static const ink_channel_t *const claria_six_color_extended_channels[] =
+{
+  &claria_black_channel,
+  &claria_cyan_channel, &extended_claria_cyan_channel,
+  &claria_magenta_channel, &extended_claria_magenta_channel,
+  &claria_yellow_channel
+};
+DECLARE_CHANNEL_SET(claria_six_color_extended);
+
+static const escp2_inkname_t claria_six_color_extended_inkset =
+{
+  "PhysicalCcMmYK", N_("Six Color Raw"), INKSET_EXTENDED,
+  &claria_six_color_extended_channel_set
+};
+
+
 static const ink_channel_t *const j_seven_color_extended_channels[] =
 {
   &photo_black_channel,
@@ -2176,14 +2449,26 @@ static const shade_set_t quadtone_shades =	/* Some kind of quadtone ink */
   { 1, { 1.0 }},
 };
 
+static const shade_set_t claria_shades =	/* Stylus R260 and newer */
+{
+  { 1, { 1.0 }},
+  { 2, { 1.0, 0.35 }},
+  { 2, { 1.0, 0.33 }},
+  { 1, { 1.0 }},
+  { 1, { 1.0 }},
+  { 1, { 1.0 }},
+  { 1, { 1.0 }},
+  { 1, { 1.0 }},
+};
+
 #define DECLARE_INKLIST(tname, name, inks, text, papers, adjustments, shades) \
 static const inklist_t name##_inklist =					      \
 {									      \
   tname,								      \
   text,									      \
   inks##_ink_types,							      \
-  &stpi_escp2_##papers##_paper_list,					      \
-  &stpi_escp2_##adjustments##_paper_adjustment_list,			      \
+  #papers,								      \
+  #adjustments,								      \
   &shades##_shades,							      \
   sizeof(inks##_ink_types) / sizeof(escp2_inkname_t *),			      \
 }
@@ -2209,6 +2494,8 @@ static const escp2_inkname_t *const standard_ink_types[] =
 
 DECLARE_INKLIST("None", standard, standard, N_("EPSON Standard Inks"),
 		standard, standard, standard);
+DECLARE_INKLIST("None", photo_gen3_4, standard, N_("EPSON Standard Inks"),
+		standard, photo3, standard);
 
 static const escp2_inkname_t *const quadtone_ink_types[] =
 {
@@ -2241,6 +2528,26 @@ static const escp2_inkname_t *const c80_quadtone_ink_types[] =
 DECLARE_INKLIST("Quadtone", c80_quadtone, c80_quadtone, N_("Quadtone"),
 		standard, standard, quadtone);
 
+static const escp2_inkname_t *const c120_ink_types[] =
+{
+  &c120_four_color_standard_inkset,
+  &c120_three_color_composite_inkset,
+  &c120_one_color_extended_inkset,
+  &c120_three_color_extended_inkset,
+  &c120_four_color_extended_inkset,
+};
+
+DECLARE_INKLIST("None", c120, c120, N_("EPSON Standard Inks"),
+		durabrite2, durabrite3, standard);
+
+static const escp2_inkname_t *const c120_quadtone_ink_types[] =
+{
+  &c120_generic_quadtone_inkset,
+};
+
+DECLARE_INKLIST("Quadtone", c120_quadtone, c120_quadtone, N_("Quadtone"),
+		standard, standard, quadtone);
+
 static const escp2_inkname_t *const c64_ink_types[] =
 {
   &c64_four_color_standard_inkset,
@@ -2351,6 +2658,22 @@ static const escp2_inkname_t *const f360_photo_ink_types[] =
 DECLARE_INKLIST("None", f360_photo, f360_photo, N_("EPSON Standard Inks"),
 		standard, sp960, esp960);
 
+static const escp2_inkname_t *const claria_ink_types[] =
+{
+  &claria_six_color_photo_inkset,
+  &claria_five_color_photo_composite_inkset,
+  &claria_four_color_standard_inkset,
+  &claria_three_color_composite_inkset,
+  &one_color_extended_inkset,
+  &claria_three_color_extended_inkset,
+  &claria_four_color_extended_inkset,
+  &claria_five_color_extended_inkset,
+  &claria_six_color_extended_inkset,
+};
+
+DECLARE_INKLIST("None", claria, claria, N_("EPSON Standard Inks"),
+		standard, claria, claria);
+
 static const escp2_inkname_t *const f360_photo7_japan_ink_types[] =
 {
   &f360_j_seven_color_enhanced_inkset,
@@ -2483,7 +2806,7 @@ DECLARE_INKLIST("picturemate", picturemate, picturemate_photo,
 
 
 #define DECLARE_INKGROUP(name)			\
-const inkgroup_t stpi_escp2_##name##_inkgroup =	\
+static const inkgroup_t name##_inkgroup =	\
 {						\
   #name,					\
   name##_group,					\
@@ -2513,6 +2836,14 @@ static const inklist_t *const c80_group[] =
 
 DECLARE_INKGROUP(c80);
 
+static const inklist_t *const c120_group[] =
+{
+  &c120_inklist,
+  &c120_quadtone_inklist
+};
+
+DECLARE_INKGROUP(c120);
+
 static const inklist_t *const c82_group[] =
 {
   &c82_inklist,
@@ -2576,6 +2907,14 @@ static const inklist_t *const photo_gen3_group[] =
 
 DECLARE_INKGROUP(photo_gen3);
 
+static const inklist_t *const photo_gen3_4_group[] =
+{
+  &photo_gen3_4_inklist,
+  &quadtone_inklist
+};
+
+DECLARE_INKGROUP(photo_gen3_4);
+
 static const inklist_t *const photo_pigment_group[] =
 {
   &pigment_inklist
@@ -2635,3 +2974,262 @@ static const inklist_t *const picturemate_group[] =
 };
 
 DECLARE_INKGROUP(picturemate);
+
+static const inklist_t *const claria_group[] =
+{
+  &claria_inklist,
+};
+
+DECLARE_INKGROUP(claria);
+
+typedef struct
+{
+  const char *name;
+  const inkgroup_t *inkgroup;
+} ink_t;
+
+static const ink_t the_inks[] =
+{
+  { "cmy", &cmy_inkgroup },
+  { "standard", &standard_inkgroup },
+  { "c120", &c120_inkgroup },
+  { "c80", &c80_inkgroup },
+  { "c82", &c82_inkgroup },
+  { "c64", &c64_inkgroup },
+  { "f360", &f360_inkgroup },
+  { "cx3650", &cx3650_inkgroup },
+  { "x80", &x80_inkgroup },
+  { "photo_gen1", &photo_gen1_inkgroup },
+  { "photo_gen2", &photo_gen2_inkgroup },
+  { "photo_gen3", &photo_gen3_inkgroup },
+  { "photo_gen3_4", &photo_gen3_4_inkgroup },
+  { "photo_pigment", &photo_pigment_inkgroup },
+  { "ultrachrome", &ultrachrome_inkgroup },
+  { "f360_photo", &f360_photo_inkgroup },
+  { "f360_photo7_japan", &f360_photo7_japan_inkgroup },
+  { "f360_ultrachrome", &f360_ultrachrome_inkgroup },
+  { "f360_ultrachrome_k3", &f360_ultrachrome_k3_inkgroup },
+  { "cmykrb", &cmykrb_inkgroup },
+  { "picturemate", &picturemate_inkgroup },
+  { "claria", &claria_inkgroup },
+};
+
+const inkgroup_t *
+stpi_escp2_get_inkgroup_named(const char *n)
+{
+  int i;
+  if (n)
+    for (i = 0; i < sizeof(the_inks) / sizeof(ink_t); i++)
+      {
+	if (strcmp(n, the_inks[i].name) == 0)
+	  return the_inks[i].inkgroup;
+      }
+  return NULL;
+}
+
+const escp2_inkname_t *
+stpi_escp2_get_default_black_inkset(void)
+{
+  return &stpi_escp2_default_black_inkset;
+}
+
+
+#define DECLARE_CHANNEL_LIST(name)			\
+static const channel_name_t name##_channel_name_list =	\
+{							\
+  #name,						\
+  sizeof(name##_channel_names) / sizeof(const char *),	\
+  name##_channel_names					\
+}
+
+static const char *standard_channel_names[] =
+{
+  N_("Black"),
+  N_("Cyan"),
+  N_("Magenta"),
+  N_("Yellow")
+};
+
+DECLARE_CHANNEL_LIST(standard);
+
+static const char *c120_channel_names[] =
+{
+  N_("Black"),
+  N_("Black"),
+  N_("Magenta"),
+  N_("Cyan"),
+  N_("Yellow")
+};
+
+DECLARE_CHANNEL_LIST(c120);
+
+static const char *cx3800_channel_names[] =
+{
+  N_("Cyan"),
+  N_("Yellow"),
+  N_("Magenta"),
+  N_("Black")
+};
+
+DECLARE_CHANNEL_LIST(cx3800);
+
+static const char *mfp2005_channel_names[] =
+{
+  N_("Cyan"),
+  N_("Magenta"),
+  N_("Yellow"),
+  N_("Black")
+};
+
+DECLARE_CHANNEL_LIST(mfp2005);
+
+static const char *photo_channel_names[] =
+{
+  N_("Black"),
+  N_("Cyan"),
+  N_("Magenta"),
+  N_("Yellow"),
+  N_("Light Cyan"),
+  N_("Light Magenta"),
+};
+
+DECLARE_CHANNEL_LIST(photo);
+
+static const char *rx700_channel_names[] =
+{
+  N_("Black"),
+  N_("Cyan"),
+  N_("Light Cyan"),
+  N_("Magenta"),
+  N_("Light Magenta"),
+  N_("Yellow"),
+};
+
+DECLARE_CHANNEL_LIST(rx700);
+
+static const char *sp1400_channel_names[] =
+{
+  N_("Yellow"),
+  N_("Black"),
+  N_("Light Cyan"),
+  N_("Light Magenta"),
+  N_("Magenta"),
+  N_("Cyan"),
+};
+
+DECLARE_CHANNEL_LIST(sp1400);
+
+static const char *sp2200_channel_names[] =
+{
+  N_("Black"),
+  N_("Cyan"),
+  N_("Magenta"),
+  N_("Yellow"),
+  N_("Light Cyan"),
+  N_("Light Magenta"),
+  N_("Light Black"),
+};
+
+DECLARE_CHANNEL_LIST(sp2200);
+
+static const char *pm_950c_channel_names[] =
+{
+  N_("Black"),
+  N_("Cyan"),
+  N_("Magenta"),
+  N_("Yellow"),
+  N_("Light Cyan"),
+  N_("Light Magenta"),
+  N_("Dark Yellow"),
+};
+
+DECLARE_CHANNEL_LIST(pm_950c);
+
+static const char *sp960_channel_names[] =
+{
+  N_("Black"),
+  N_("Cyan"),
+  N_("Magenta"),
+  N_("Yellow"),
+  N_("Light Cyan"),
+  N_("Light Magenta"),
+  N_("Black"),
+};
+
+DECLARE_CHANNEL_LIST(sp960);
+
+static const char *r800_channel_names[] =
+{
+  N_("Yellow"),
+  N_("Magenta"),
+  N_("Cyan"),
+  N_("Matte Black"),
+  N_("Photo Black"),
+  N_("Red"),
+  N_("Blue"),
+  N_("Gloss Optimizer"),
+};
+
+DECLARE_CHANNEL_LIST(r800);
+
+static const char *picturemate_channel_names[] =
+{
+  N_("Yellow"),
+  N_("Magenta"),
+  N_("Cyan"),
+  N_("Black"),
+  N_("Red"),
+  N_("Blue"),
+};
+
+DECLARE_CHANNEL_LIST(picturemate);
+
+static const char *r2400_channel_names[] =
+{
+  N_("Light Light Black"),
+  N_("Light Magenta"),
+  N_("Light Cyan"),
+  N_("Light Black"),
+  N_("Black"),
+  N_("Cyan"),
+  N_("Magenta"),
+  N_("Yellow"),
+};
+
+DECLARE_CHANNEL_LIST(r2400);
+
+typedef struct
+{
+  const char *name;
+  const channel_name_t *channel_name;
+} channel_t;
+
+static const channel_t the_channels[] =
+{
+  { "cx3800", &cx3800_channel_name_list },
+  { "mfp2005", &mfp2005_channel_name_list },
+  { "photo", &photo_channel_name_list },
+  { "picturemate", &picturemate_channel_name_list },
+  { "pm_950c", &pm_950c_channel_name_list },
+  { "r2400", &r2400_channel_name_list },
+  { "sp1400", &sp1400_channel_name_list },
+  { "r800", &r800_channel_name_list },
+  { "rx700", &rx700_channel_name_list },
+  { "sp2200", &sp2200_channel_name_list },
+  { "sp960", &sp960_channel_name_list },
+  { "standard", &standard_channel_name_list },
+  { "c120", &c120_channel_name_list },
+};
+
+const channel_name_t *
+stpi_escp2_get_channel_names_named(const char *n)
+{
+  int i;
+  if (n)
+    for (i = 0; i < sizeof(the_channels) / sizeof(channel_t); i++)
+      {
+	if (strcmp(n, the_channels[i].name) == 0)
+	  return the_channels[i].channel_name;
+      }
+  return NULL;
+}
